##########################################################################
# File         : tsiControlPanel.tcl
# Author       : Gordon D. Baxter
# Date	       : 01/07/96
# Description  : 
#
#
# This version of the control panel was primarily written by Randy Jones.
# 
# $Id: tsiControlPanel.tcl,v 8.1 1998/09/18 16:42:34 swallace Exp $
# $Log: tsiControlPanel.tcl,v $
# Revision 8.1  1998/09/18 16:42:34  swallace
# Works fine,, but dirty.  About to clean up.
#
# Revision 7.2  1998/08/21 12:11:44  swallace
# seems to work w/ remote agent proprioceptic feedback
#
# Revision 7.1  1998/06/23 13:49:50  swallace
# Works w/ eaters in Run Mode & Step Mode
# Works w/ CLIPS
#
# Revision 5.1  1998/06/02 16:50:48  swallace
# 2 side step & run
#
# Revision 4.1  1998/06/01 22:14:08  swallace
# works using step on both sides.
#
# Revision 3.2  1998/05/28 20:17:57  swallace
# Ready to work on WME wrappers.
#
# Revision 3.1  1998/05/21 18:36:55  swallace
# Client/Server Interface allows info to be passed to client.
#
# Revision 2.2  1998/05/19 21:42:31  swallace
# slightly better than previous version
#
# Revision 2.1  1998/05/19 17:21:56  swallace
# *** empty log message ***
#
# Revision 1.3  1998/05/19 05:03:31  swallace
# works, but soar is not completely seperated.
#
# Revision 1.2  1998/05/19 02:01:44  swallace
# Version 1.2 -- semi-complete desn't crash
#
#
##########################################################################

###
###	I.	Make the TSI Control Panel
###


proc makeTSIControlPanel { {hide 0} {agentCreationGUI tsiTextEntryACGUI} } {
  global tsi_library tsiConfig


  set tsiConfig(agentCreationGUI) $agentCreationGUI

  if $hide {
    wm withdraw .
  } else {
    wm deiconify .
  }
  . configure -relief ridge -borderwidth 5
  wm title . {Agent Control Panel}

  set x $tsiConfig(tsiControlPanelX)
  set y $tsiConfig(tsiControlPanelY)


  if {$x >= 0} { set x +$x }
  if {$y >= 0} { set y +$y }
  
  wm geometry . $x$y

  # create the menu bar
  frame .menu -relief raised -border 2
  pack .menu -side top -fill x

  menubutton .menu.file -text {File} -menu .menu.file.m

  menu .menu.file.m 
  .menu.file.m add command -label Exit -command quitSoar

  pack .menu.file -side left -padx 5

  # create a frame for the general buttons

  frame .run -borderwidth 2
  button .run.step -text Step -command environmentStep
  button .run.run -text Run -command environmentRun 
  button .run.stop -text Stop -command environmentStop
  button .run.quit -text Quit -command quitSoar
  

  # pack the frame, and its components
  pack .run.step .run.run .run.stop .run.quit -side top -anchor n -fill both
  pack .run -side right -fill y

  frame .agents

  # Call the agentCreationGUI procedure...  
#  $agentCreationGUI .agents
    tsiPullDownMenuACGUI .agents

  # The rest of the window will be filled with agent frames as they
  # are registered with the control panel.
  # There will be a permanent frame at the top for commands to be
  # issued to all agents at the same time

  frame .agents.agents -relief ridge -borderwidth 5
  pack .agents.agents -side top -fill x

  label .agents.agents.label -text {Agent Menus:}
  menubutton .agents.agents.all -text {All Agents} -relief raised \
             -menu .agents.agents.all.m

  menu .agents.agents.all.m
  .agents.agents.all.m add command -label {Open All Agent Windows} \
       -command {foreach a [tsiListAgents] { \
                    set showAgent($a) 1 ;\
                    tsiSendAgent $a {wm deiconify .tsw} \
                 }}
  .agents.agents.all.m add command -label {Close All Agent Windows} \
       -command {foreach a [tsiListAgents] { \
                    set showAgent($a) 0 ;\
                    tsiSendAgent $a {wm withdraw .tsw} \
                 }}
  .agents.agents.all.m add command -label {Watch 0} \
       -command {foreach a [tsiListAgents] { \
                    tsiSendAgent $a {tsiDisplayAndSendCommand {watch 0}} \
                 }}
  .agents.agents.all.m add separator
  .agents.agents.all.m add command -label {Clear Working Memory} \
         -command {sendAllAgents init-soar}
  .agents.agents.all.m add command -label {Excise All} \
         -command {sendAllAgents {excise -all}}
  .agents.agents.all.m add command -label {Excise Chunks} \
         -command {sendAllAgents {excise -chunks}}

  pack .agents.agents.label .agents.agents.all -side left -fill y
  pack .agents -side top -padx 10 -pady 10


  # Now we need to do the specialized setup for each particular mode

    if { ![string compare $tsiConfig(mode) client] } {
      # Client mode
      
      uplevel #0 [list source [file join $tsi_library tsi-client.tcl]]
      tsiInitClientControlPanel $tsiConfig(socketioHost) \
	  $tsiConfig(socketioPort)

  } elseif { ![string compare $tsiConfig(mode) server] } {
      # Server mode
      
      
      uplevel #0 [list source [file join $tsi_library tsi-server.tcl]]
      tsiInitServerControlPanel $tsiConfig(socketioPort)

  } else {
      # Socketio Off

      uplevel #0 [list source [file join $tsi_library tsi-nosio.tcl]]
           
      
  }


}


proc tsiTextEntryACGUI { agentFrame } {


   frame $agentFrame.create  -borderwidth 2
   label $agentFrame.create.name -text {Name:}

   entry $agentFrame.create.agentNameEntry -width 20 \
       -relief sunken -textvariable newAgentName
   button $agentFrame.create.createAgentButton \
       -text {Create New Agent} -relief raised \
       -command {createNewAgent $newAgentName; set newAgentName {}}
   

   pack $agentFrame.create.name $agentFrame.create.agentNameEntry \
        $agentFrame.create.createAgentButton -side left
   pack $agentFrame.create -side top -fill x


   ## no click to focus
   bind . <Enter> { focus $agentFrame.create.agentNameEntry }
   bind . <Return> { $agentFrame.create.agentCreationButton invoke }

}

proc tsiPullDownMenuACGUI { agentFrame } {

    frame $agentFrame.create -borderwidth 2
    label $agentFrame.create.dirLabel -text {Directory: }
    
    tsiLoadAgentNames $agentFrame.create.files
    tsiLoadSubDirectories $agentFrame.create.subdirs $agentFrame.create.files
    
    label $agentFrame.create.fileLabel -text {File: }

    button $agentFrame.create.createAgentButton -text {Create New Agent} \
	-relief raised -command {createNewAgent $tsiCurrentAgentSourceFile}

    pack $agentFrame.create -side top -fill x
    pack $agentFrame.create.dirLabel $agentFrame.create.subdirs \
	$agentFrame.create.fileLabel $agentFrame.create.files \
	$agentFrame.create.createAgentButton -side left
}

proc tsiLoadSubDirectories { menuName fileMenuName } {
    global tsiSimulatorPath tsiCurrentAgentSourceDir

    
    if { $tsiSimulatorPath == [pwd] } {
	set dirs [list .]
	foreach file [glob -nocomplain *] {
	    if { [file isdirectory $file] } {
		lappend dirs [file join . [file tail $file]]
	    }
	}

    } else {
	set currDir [file join . [file tail [pwd]]]
	set dirs [list $currDir [file join $currDir ..]]
    }
       
    if [winfo exists $menuName] {
	[$menuName cget -menu] delete 0 end
	foreach i $dirs {
	    puts "Adding $i"
	    [$menuName cget -menu] add command -label $i \
		-command [list tsiChangeAgentSourceDir $i $menuName $fileMenuName]
	}

	set tsiCurrentAgentSourceDir [lindex $dirs 0]
	
    } else {
	eval tk_optionMenu $menuName tsiCurrentAgentSourceDir $dirs
	foreach dir $dirs {
	    [$menuName cget -menu] entryconfigure $dir -command \
		[list tsiChangeAgentSourceDir $dir $menuName $fileMenuName]
	}
    }
    
}

proc tsiChangeAgentSourceDir { newDir dirMenuName fileMenuName } {
    global tsiSimulatorPath

    set oldDir [pwd]
    cd [file join $tsiSimulatorPath $newDir]

    if { $oldDir == [pwd] } { return }

    tsiLoadAgentNames $fileMenuName
    tsiLoadSubDirectories $dirMenuName $fileMenuName


}

    

proc tsiLoadAgentNames { menuName } {
    global tsiCurrentAgentSourceFile 

    set allNames [glob -nocomplain {*.soar}]
    set names ""
    
    foreach i $allNames {
	lappend names [file rootname [file tail $i]]
    }

    if { $names == {} } {
	
	if [winfo exists $menuName] {
	    [$menuName cget -menu] delete 0 end
	    [$menuName cget -menu] add command -label {*NONE*} -command {}
	} else {
	    tk_optionMenu $menuName {*NONE*}
	    [$menuName cget -menu] entryconfigure {*NONE*} -command {}
	}
	$menuName configure -state disabled
	set currentName {}
    } else {
	if [winfo exists $menuName] {
	    [$menuName cget -menu] delete 0 end
	    foreach i $names {
		[$menuName cget -menu] add command -label $i \
		    -command [list set tsiCurrentAgentSourceFile $i]
	    }
	    if { ![info exists currentName] || \
		     ([lsearch $names $tsiCurrentAgentSourceFile] < 0)} {
		set tsiCurrentAgentSourceFile [lindex $names 0]
	    } 
	    $menuName configure -state normal
	} else {
	    eval tk_optionMenu $menuName tsiCurrentAgentSourceFile $names
	}
    }
}



###
###	II.	Helper functions
###

### For Soar 7.1, assume that all agents are in slave interpreters
### to the controller.  So if people try to get creative with their
### interpreter hierarchies, this will fail.
proc tsiListAgents {} {
   if [catch {set x [interp slaves]} m] {
      error "Cannot compute list of agents: $m"
   }
   return $x
}

### need to worry about having whitespace in the agent name...that
### can break things RMJ
proc createNewAgent {name {filepath ""} {filename ""}} {
   global auto_path tsi_library soar_library soar_doc_dir tsiConfig \
       localAgents
   
   if [catch "interp create $name"] {
      error "Cannot create agent named '$name'"
   }

    if [info exists localAgents] { 
	lappend localAgents $name 
    } else {
	set localAgents $name
    }

   load {} Tk $name
   $name eval [list set auto_path $auto_path]
   $name eval {package require Soar}
   $name eval [list set soar_library $soar_library]
   $name eval [list set tsi_library $tsi_library]
   if [info exists soar_doc_dir] {
      $name eval [list set soar_doc_dir $soar_doc_dir]
   }
   ### Configure the agent for the TSI
   set id [array startsearch tsiConfig]
   while {[array anymore tsiConfig $id]} {
      set elm [array nextelement tsiConfig $id]
      $name eval [list set tsiConfig($elm) $tsiConfig($elm)]
   }
   array donesearch tsiConfig $id
   $name eval [list set interp_name $name]
   $name eval [list set source_path $filepath]
   $name eval [list set source_file $filename]
   $name alias registerWithController registerAgent $name
   $name alias tsiListAgents tsiListAgents
   $name eval tsiInitAgent
   
   tsiSetupAgentVars $name
 

}

proc tsiSendAgent {agent command} {
   ### Need to use {} for catch, or $command gets concat'ed into a list
   if [catch {$agent eval $command} msg] {
      error "Error sending command '$command' to agent $agent!"
   }
}

### New version using slave interpreters in Soar 7.1
proc sendAllAgents {msg} {
   foreach i [tsiListAgents] {
      if [catch {$i eval $msg} m] {
         error "Error sending message $msg to agent $i: $m"
      }
   }
}



proc registerAgent {name} {
  global showAgent tsiConfig sio_agentStatus tsiAgentInfo

   puts "registerAgent   (begin) -- name: $name"
   

  menubutton .agents.agents.agent$name -text $name -relief raised \
              -menu .agents.agents.agent$name.m

  menu .agents.agents.agent$name.m

  # if this is a ClientTSI interface, we add an additional checkbutton
  # to each agent menu.
    if { ![string compare $tsiConfig(mode) client] } {
	.agents.agents.agent$name.m add checkbutton -label {Online} \
	   -variable sio_agentStatus($name,online) \
	   -command "agentToggleCommand $name \$sio_agentStatus($name,online) \
                       {SetClientOnline 1} {SetClientOnline 0}"
    }
	
  .agents.agents.agent$name.m add checkbutton -label {Show Agent Window} \
       -variable showAgent($name) \
       -command "agentToggleCommand $name \$showAgent($name) \
                   {wm deiconify .tsw} {wm withdraw .tsw}"
  set showAgent($name) 1
  .agents.agents.agent$name.m add command -label {Raise Agent Window} \
       -command "tsiSendAgent $name {raise .tsw}"
  .agents.agents.agent$name.m add command -label {watch 0} \
      -command "tsiSendAgent $name {tsiDisplayAndSendCommand {watch 0}}"
  .agents.agents.agent$name.m add separator
  .agents.agents.agent$name.m add command -label {Clear Working Memory} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand init-soar}"
  .agents.agents.agent$name.m add command -label {Excise All} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand {excise -all}}"
  .agents.agents.agent$name.m add command -label {Excise Chunks} \
         -command "tsiSendAgent $name {tsiDisplayAndSendCommand {excise -chunks}}"
  .agents.agents.agent$name.m add separator
  .agents.agents.agent$name.m add command -label {Destroy Agent} \
      -command "destroyAgent $name"

  pack .agents.agents.agent$name -side left -fill y

   set tsiAgentInfo($name) local
   set tsiAgentInfo($name,addFN) localAgentAddWME
   set tsiAgentInfo($name,rmFN) localAgentRemoveWME
   
  
}

proc agentToggleCommand {name t on off} {
   if $t {
      tsiSendAgent $name $on
   } else {
      tsiSendAgent $name $off
   }
}

proc destroyAgent {name} {
   if {[lsearch [tsiListAgents] $name] >= 0} {
      if [catch "destroy-interpreter $name"] {
         if [catch "interp delete $name"] {
            error "Cannot delete agent '$name'!"
         }
      }
   }
   if [winfo exists .agents.agents.agent$name] {
      destroy .agents.agents.agent$name
   }
}
